import functools


try:
    import newrelic.agent as agent

    def set_transaction_name(func):
        @functools.wraps
        def inner(request, resource_path):
            agent.set_transaction_name(
                resource_path,
                'Python/WebFramework/Controller'
            )
            return func(request, resource_path)
        return inner

except ImportError:
    def set_transaction_name(func):
        return func



from collections import OrderedDict
import logging
import urllib

import dirty_bits
import django.core.exceptions

from savory_pie.django.fields import ReverseField
from savory_pie.django.utils import Related
from savory_pie.django.validators import ValidationError, validate
from savory_pie.errors import SavoryPieError
from savory_pie.helpers import get_sha1
from savory_pie.resources import EmptyParams, Resource

logger = logging.getLogger(__name__)


class QuerySetResource(Resource):
    """
    Resource abstract around Django QuerySets.

    Parameters:

        ``resource_class``
            type of Resource to create for a given Model in the queryset

    Typical usage...

    .. code::

        class FooResource(ModelResource):
            parent_resource_path = 'foos'
            model_class = Foo

        class FooQuerySetResource(QuerySetResource):
            resource_class = FooResource
    """
    #: optional - if set specifies the page size for data returned during a GET
    #: - defaults to None (no paging)
    page_size = None
    filters = []

    #Setup so that by default we will allow Unfiltered Queries
    #TODO: We need to swap this to False eventually and whitelist. However to limit halo, a blacklist will be used.
    allow_unfiltered_query = True

    def __init__(self, queryset=None):
        if queryset is not None:
            self.queryset = queryset
        else:
            self.queryset = self.resource_class.model_class.objects.all()

    @property
    def supports_paging(self):
        return self.page_size is not None

    @property
    def resource_path(self):
        return self.resource_class.parent_resource_path

    def filter_queryset(self, ctx, params, queryset):
        for filter in self.filters:
            queryset = filter.filter(ctx, params, queryset)

        # The extra filter call exists to keep a test passing
        return queryset.filter()

    def slice_queryset(self, ctx, params, queryset):
        if self.supports_paging:
            page = params.get_as('page', int, 0)
            offset = page * self.page_size
            return queryset[offset: offset + self.page_size]
        else:
            return queryset

    def build_page_uri(self, ctx, page):
        return ctx.build_resource_uri(self) + '?' + urllib.urlencode({'page': page})

    def to_resource(self, model):
        """
        Constructs a new instance of resource_class around the provided model.
        """
        resource = self.resource_class(model)

        # Normally, traversal would take care of filling in the resource_path
        # for a child resource, but this is called to create sub-resources that are
        # embedded into a larger GET.  To make sure, the resourceUri can be
        # computed for those resources, we need to make sure they have a resource_path.
        if resource.resource_path is None and self.resource_path is not None:
            resource.resource_path = self.resource_path + '/' + str(resource.key)

        return resource

    @classmethod
    def prepare(cls, ctx, related):
        cls.resource_class.prepare(ctx, related)

    def prepare_queryset(self, ctx, queryset):
        related = Related()
        self.prepare(ctx, related)
        return related.prepare(queryset)

    def has_valid_key(self, ctx, params):
        get_query_dict = getattr(params, '_GET', None)
        if get_query_dict:
            param_keys = set(get_query_dict.keys())
            filter_query_args = {ctx.formatter.convert_to_public_property(filter.name) for filter in self.filters}
            if not filter_query_args:
                return True
            for paramkey in param_keys:
                if paramkey in filter_query_args:
                    # At least one param key provided is in the resources list of filters, therefore valid
                    return True
        return False

    def get(self, ctx, params):
        if not self.allow_unfiltered_query and not self.has_valid_key(ctx, params):
            raise SavoryPieError(
                'Request must be filtered, will not return all.  Acceptable filters are: {0}'.format([filter.name for filter in self.filters])
            )

        complete_queryset = self.queryset.all().distinct()

        filtered_queryset = self.filter_queryset(ctx, params, complete_queryset)
        sliced_queryset = self.slice_queryset(ctx, params, filtered_queryset)

        # prepare must be last for optimization to be respected by Django.
        final_queryset = self.prepare_queryset(ctx, sliced_queryset)

        objects = []
        for model in final_queryset:
            model_json = self.to_resource(model).get(ctx, EmptyParams())
            model_json['$hash'] = get_sha1(ctx, model_json)
            objects.append(model_json)

        meta = dict()
        if self.supports_paging:
            # When paging the sliced_queryset will not contain all the objects,
            # so the count of the accumulated objects is insufficient.  In that case,
            # need to make a call to queryset.count.
            count = filtered_queryset.count()

            page = params.get_as('page', int, 0)
            if page > 0:
                meta['prev'] = self.build_page_uri(ctx, page - 1)

            meta['count'] = count

            if (page + 1) * self.page_size < count:
                meta['next'] = self.build_page_uri(ctx, page + 1)
        else:
            # When paging is disabled the sliced_queryset is the complete queryset,
            # so the accumulated objects contains all the objects.  In this case, just
            # do a len on the accumulated objects to avoid the extra COUNT(*) query.
            meta['count'] = len(objects)

        # add meta-level resourceUri to QuerySet response
        if self.resource_path is not None:
            meta['resourceUri'] = ctx.build_resource_uri(self)

        return {
            'meta': meta,
            'objects': objects
        }

    def post(self, ctx, source_dict):
        resource = self.resource_class.create_resource()
        if filter(lambda field: isinstance(field, ReverseField), resource.fields):
            raise ValidationError(resource, {'do not post a resource with a ReverseField':
                                             type(resource).__name__})
        with ctx.target(resource.model):
            resource.put(ctx, source_dict)

        # If the newly created child_resource is not absolutely addressable on
        # its own, then fill in the address (assuming the QuerySetResource
        # is addressable itself.)
        if resource.resource_path is None and self.resource_path is not None:
            resource.resource_path = self.resource_path + '/' + str(resource.key)

        return resource

    def get_child_resource(self, ctx, path_fragment):
        if path_fragment == 'schema':
            return SchemaResource(self.resource_class)

        # No need to filter or slice here, does not make sense as part of get_child_resource
        queryset = self.prepare_queryset(ctx, self.queryset)
        try:
            model = self.resource_class.get_from_queryset(queryset, path_fragment)
            return self.to_resource(model)
        except queryset.model.DoesNotExist:
            return None


class DirtyInitializerMetaClass(type):

    def __new__(cls, name, bases, dct):
        model_class = dct.get('model_class', None)
        if model_class:
            dirty_bits.register(model_class)
        else:
            for base in bases:
                model_class = getattr(base, 'model_class', None)
                if model_class:
                    dirty_bits.register(model_class)
                    break
        return type.__new__(cls, name, bases, dct)


class ModelResource(Resource):
    """
    Resource abstract around ModelResource.

    Typical usage...

    .. code::

        class FooResource(ModelResource):
            parent_resource_path = 'foos'
            model_class = Foo

        class FooQuerySetResource(QuerySetResource):
            resource_class = FooResource
    """
    # Must have a metaclass to instantiate dirty checking
    __metaclass__ = DirtyInitializerMetaClass

    #: path of parent resource - used to compute resource_path
    parent_resource_path = None

    #: tuple of (name, type) of the key property used in the resource_path
    published_key = ('pk', int)

    #: A list of Field-s that are used to determine what properties are placed
    #: into and read from dict-s being handled by get, post, and put
    fields = []

    #: A list of Validator-s that are used to check data consistency and
    #: integrity on a model.
    validators = []

    _resource_path = None

    @classmethod
    def get_from_queryset(cls, queryset, path_fragment):
        """
        Called by containing QuerySetResource to filter the QuerySet down
        to a single item -- represented by this ModelResource
        """
        attr, type_ = cls.published_key

        kwargs = dict()
        kwargs[attr] = type_(path_fragment)
        return queryset.get(**kwargs)

    @classmethod
    def create_resource(cls):
        """
        Creates a new ModelResource around a new model_class instance
        """
        return cls(cls.model_class())

    @classmethod
    def prepare(cls, ctx, related):
        """
        Called by QuerySetResource to add necessary select_related-s
        calls to the QuerySet.
        """
        for field in cls.fields:
            try:
                prepare = field.prepare
            except AttributeError:
                pass
            else:
                prepare(ctx, related)
        return related

    @classmethod
    def get_by_source_dict(cls, ctx, source_dict):
        filters = {}
        for field in cls.fields:
            try:
                filter_by_item = field.filter_by_item
            except AttributeError:
                pass
            else:
                filter_by_item(ctx, filters, source_dict)

        try:
            model = cls.model_class.objects.filter(**filters).get()
        except django.core.exceptions.ObjectDoesNotExist:
            return None
        else:
            return cls(model)

    def __init__(self, model):
        self.model = model

    @property
    def key(self):
        """
        Provides the value of the published_key of this ModelResource.
        May fail if the ModelResource was constructed around an uncommitted Model.
        """
        attr, type_ = self.published_key
        return str(getattr(self.model, attr))

    @property
    def resource_path(self):
        if self._resource_path is not None:
            return self._resource_path
        elif self.parent_resource_path is not None:
            return self.parent_resource_path + '/' + str(self.key)
        else:
            return None

    @resource_path.setter
    def resource_path(self, resource_path):
        # TODO: Sanity checks that path is bound properly
        self._resource_path = resource_path

    def get(self, ctx, params):
        target_dict = OrderedDict()

        for field in self.fields:
            field.handle_outgoing(ctx, self.model, target_dict)

        if self.resource_path is not None:
            target_dict['resourceUri'] = ctx.build_resource_uri(self)

        return target_dict

    def _set_pre_save_fields(self, ctx, source_dict):
        for field in self.fields:
            try:
                pre_save = field.pre_save
            except AttributeError:
                field.handle_incoming(ctx, source_dict, self.model)
            else:
                if pre_save(self.model):
                    field.handle_incoming(ctx, source_dict, self.model)

    def _set_post_save_fields(self, ctx, source_dict):
        for field in self.fields:
            try:
                pre_save = field.pre_save
            except AttributeError:
                pass
            else:
                if not pre_save(self.model):
                    field.handle_incoming(ctx, source_dict, self.model)

    def _save(self):
        if self.model.is_dirty():
            self.model.save()

        for field in self.fields:
            try:
                save = field.save
            except AttributeError:
                pass
            else:
                save(self.model)

    def put(self, ctx, source_dict, save=True, skip_validation=False):
        '''
        This is where we respect the 'pre_save' flag on each field.
        If pre_save is true, then we set the field value, before calling save.
        If not, call save first, before setting the field value, this is for the
        many-to-many relationship.
        '''
        if not source_dict:
            return

        if not skip_validation:
            errors = validate(ctx, self.__class__.__name__, self, source_dict)
            if errors:
                logger.debug(errors)
                raise ValidationError(self, errors)

        try:
            self._set_pre_save_fields(ctx, source_dict)
        except TypeError, e:
            import traceback
            for L in traceback.format_exc().splitlines():
                logger.debug(L)
            raise ValidationError(self, {'invalidFieldData': e.message})

        if save:
            self._save()
            logger.debug('save succeeded for %s' % self)

        self._set_post_save_fields(ctx, source_dict)
        logger.debug('put succeeded for %s' % self)

    def delete(self, ctx):
        self.model.delete()

    @classmethod
    def _validator_schema(cls):
        return [validator.to_schema() for validator in cls.validators]


class SchemaResource(Resource):
    def __init__(self, model_resource):
        self.__resource = model_resource

    @property
    def allowed_methods(self):
        return self.__resource(self.__resource.model_class).allowed_methods

    def get(self, ctx, params=None, **kwargs):
        schema = {
            'allowedDetailHttpMethods': [m.lower() for m in self.allowed_methods],
            'allowedListHttpMethods': [m.lower() for m in self.allowed_methods],
            'defaultFormat': getattr(self.__resource, 'default_format', 'application/json'),
            'defaultLimit': getattr(self.__resource, 'default_limit', 0),
            'filtering': getattr(self.__resource, 'filtering', {}),
            'ordering': getattr(self.__resource, 'ordering', []),
            'validators': self.__resource._validator_schema(),
            'resourceUri': ctx.build_resource_uri(self),
            'fields': {},
        }
        for resource_field in self.__resource.fields:
            try:
                field_name = ctx.formatter.convert_to_public_property(resource_field.name)
                schema['fields'][field_name] = resource_field.schema(ctx, model=self.__resource.model_class)
            except AttributeError:
                pass
        return schema



from savory_pie.errors import AuthorizationError


def authorization_adapter(field, ctx, source_dict, target_obj):
    """
    Default adapter works on single field (non iterable)
    """
    name = field._compute_property(ctx)
    source = field.to_python_value(ctx, source_dict[name])
    target = field.to_python_value(ctx, field._get(target_obj))
    return name, source, target


def datetime_auth_adapter(field, ctx, source_dict, target_obj):
    """
    Adapter for fields of date/datetime/time
    """
    name = field._compute_property(ctx)
    source = field.to_python_value(ctx, source_dict[name])
    target = field._get(target_obj)
    return name, source, target


def subobject_auth_adapter(field, ctx, source_dict, target_obj):
    """
    Adapter for fields of savory_pie.fields.SubObjectResourceField, or subclasses thereof
    """
    name = field._compute_property(ctx)
    if source_dict[name] is not None:
        source = source_dict[name]['resourceUri']
    else:
        source = None
    # this is essentially the same logic as in field.get_subresource(), but
    # ignores source_dict as we're only interested in target's resourceUri
    target_subobject = getattr(target_obj, field.name)
    if target_subobject is not None:
        target = ctx.build_resource_uri(field._resource_class(target_subobject))
    else:
        target = None
    return name, source, target


def uri_auth_adapter(field, ctx, source_dict, target_obj):
    """
    Authorization adapter for use in fields representing a 1 to many relationship.  Is used when you want to prevent
    unauthorized users from changing the associations of different models.
    """
    name = field._compute_property(ctx)
    source_field = source_dict[name]
    target_field = getattr(target_obj, field.name)

    from savory_pie.fields import URIResourceField, URIListResourceField, IterableField

    if source_field and target_field:
        if isinstance(field, IterableField):
            source = [source_field_item.get('resourceUri', None) for source_field_item in source_field]
            target = [ctx.build_resource_uri(field._resource_class(target_item))
                      for target_item in field.get_iterable(target_field)]
            source.sort()
            target.sort()
        elif isinstance(field, URIResourceField):
            source = source_field
            target = ctx.build_resource_uri(field._resource_class(target_field))
        elif isinstance(field, URIListResourceField):
            source = source_field
            target = [ctx.build_resource_uri(field._resource_class(target_item))
                      for target_item in field.get_iterable(target_field)]
            source.sort()
            target.sort()
        else:
            raise TypeError('uri_auth_adapter can only be used with fields of type URIResourceField,' +
                            ' URIListResourceField or IterableField')
    else:
        source = None
        target = None

    return name, source, target


class authorization(object):
    """
    Authorization decorator, takes a permission dictionary key and an adapter function
    @auth_adapter: an adapter function that takes ctx, source_dict, target_obj and
        returns ctx, target_obj, source, target parameters

        Use:
            @authorization(adapter)

    """
    def __init__(self, auth_adapter):
        self.auth_adapter = auth_adapter

    def __call__(self, fn):
        """
        If the user does not have an the authorization raise an AuthorizationError
        """
        def inner(field, ctx, source_dict, target_obj):
            permission = field.permission
            if permission:
                auth_adapter = getattr(permission, 'auth_adapter', None) or self.auth_adapter
                name, source, target = auth_adapter(field, ctx, source_dict, target_obj)
                if not permission.is_write_authorized(ctx, target_obj, source, target):
                    raise AuthorizationError(name)

            return fn(field, ctx, source_dict, target_obj)

        return inner



import contextlib


class APIContext(object):
    """
    Context object passed as the second argument (after self) to Resources and Fields.

    The context object provides a hook into the underlying means to translates
    resources to / from URIs.

    The context has a streaming_response attribute which defaults to False. If
    this is set to True the get method of the resource should not return a
    dict, but an iterable of strings. It is the job of the resource to make
    sure the content_type of the request, ctx.formatter.content_type is
    respected. This can be used as a performance improvement when returning
    large result sets where fragments of them can be pre-computed/cached and
    stitched in to a final result.
    """
    def __init__(self, base_uri, root_resource, formatter, request=None):
        self.base_uri = base_uri
        self.root_resource = root_resource
        self.formatter = formatter
        self.request = request
        self.expiration = None
        self._headers_dict = {}
        self.object_stack = []
        self.streaming_response = False

    def resolve_resource_uri(self, uri):
        """
        Resolves the resource that corresponds to the current URI,
        but only within the same resource tree.
        """
        if not uri.startswith(self.base_uri):
            return None

        return self.resolve_resource_path(uri[len(self.base_uri):])

    def resolve_resource_path(self, resource_path):
        """
        Resolves a resource using a resource path (not a full URI),
        but only within the same resource tree.
        """
        resource = self.root_resource
        cur_resource_path = ''

        for path_fragment in _split_resource_path(resource_path):
            cur_resource_path = cur_resource_path + '/' + path_fragment
            resource = resource.get_child_resource(self, path_fragment)

            if not resource:
                return None

            if resource.resource_path is None:
                resource.resource_path = cur_resource_path

        return resource

    def build_resource_uri(self, resource):
        """
        Given a Resource with a resource_path, provides the correspond URI.
        Raises a ValueError if the resource_path of the Resource is None.
        """
        resource_path = resource.resource_path

        if resource_path is None:
            raise ValueError(u'unaddressable resource')

        return self.base_uri + resource_path

    def set_header(self, header, value):
        """
        Updates self.header_dict property for processing in the view where the Response headers should be set from
        header_dict
        """
        self._headers_dict[header] = value
        return self._headers_dict

    @property
    def headers(self):
        if self.expiration:
            self.set_header('Expires', self.expiration.isoformat('T'))
        return self._headers_dict

    def set_expires_header(self, new_expiration):
        """
        Keeps a min expiration in memory and sets it on header request
        """
        self.expiration = new_expiration if not self.expiration else min(self.expiration, new_expiration)

    @contextlib.contextmanager
    def target(self, target):
        self.push(target)
        yield
        self.pop()

    def push(self, target):
        self.object_stack.append(target)

    def pop(self):
        return self.object_stack.pop()

    def peek(self, n=1):
        return self.object_stack[-n]


def _split_resource_path(resource_path):
    path_fragments = resource_path.split('/')
    if path_fragments[-1] == '':
        return path_fragments[:-1]
    else:
        return path_fragments



import haystack.inputs

from haystack.query import SearchQuerySet

from savory_pie.resources import Resource
from savory_pie.helpers import _hash_string


class HaystackSearchResource(Resource):
    """
    The HaystackSearchResource returns model API details directly from the
    search index. This is much faster than making the DB queries, but can
    result in old data being returned.
    """
    model_class = None

    def _filter_qs(self, params, qs):
        return qs

    def get(self, ctx, params):
        ctx.streaming_response = True
        qs = SearchQuerySet().models(self.model_class)
        if 'q' in params:
            qs = qs.filter(content=haystack.inputs.AutoQuery(params.get('q')))
        qs = self._filter_qs(params, qs)

        def result(qs):
            # Use a result generator so we can avoid string concatenation
            count = qs.count()
            last = count - 1
            yield '{"meta":{"count":'
            yield str(count)
            yield '},"objects":['
            for i, result in enumerate(qs):
                # TODO document this ugliness
                apistring = result.get_stored_fields()['api']
                apistring = apistring.replace('SAVORY_PIE_HOSTNAME', ctx.base_uri)

                if apistring.endswith('}'):
                    apistring = '{},"$hash":"{}"}}'.format(
                        apistring[:-1],
                        _hash_string(apistring)
                    )
                yield apistring
                if i != last:
                    yield ','
            yield ']}'
        return result(qs)



from datetime import datetime


def to_datetime(milliseconds):
    """
    Converts milliseconds in UTC (e.g., from JS `new Date().getTime()` into Python datetime
    """
    try:
        value = datetime.utcfromtimestamp(int(milliseconds) / 1000)
        if isinstance(value, datetime):
            return value
    except:
        pass
    return milliseconds


def to_list(items):
    """
    Converts comma-delimited string into list of items
    """
    try:
        values = items.split(',')
        if isinstance(values, list):
            return values
    except:
        pass
    return items


class ParamsDict(object):
    """
    Simple class that wraps a dictionary and returns a list.
    This is used because filters support getting a list of values given a parameter,
    so when using a filter within a queryset, the filter (in the format of a dict)
    needs to support list related functions, this class acts a wrapper around the filter.

        Parameters:

            ``params``
                This is a dictionary of parameters
    """
    def __init__(self, params):
        self._params = params

    def keys(self):
        return self._params.keys()

    def __contains__(self, key):
        return key in self._params

    def __getitem__(self, key):
        if key in self._params:
            return self._params.get(key)
        else:
            raise KeyError

    def get(self, key, default=None):
        return self._params.get(key, default)

    def get_as(self, key, type, default=None):
        value = self._params.get(key, None)
        return default if value is None else type(value)

    def get_list(self, key):
        return [self._params[key]]

    def get_list_of(self, key, type):
        list = self._params.get(key, None)
        if list is None:
            return []
        else:
            return [type(x) for x in list]



import operator
import random

from django.db.models import Q
from django.db.models.signals import post_init

from mock import Mock
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist


class QuerySet(Mock):
    def __init__(self, *elements):
        super(QuerySet, self).__init__()
        if elements:
            self.model = type(elements[0])
        else:
            self.model = Model

        self._elements = elements
        self._selected = set()
        self._prefetched = set()

    def __iter__(self):
        return self.iterator()

    def iterator(self):
        return iter(self._elements)

    def __list__(self):
        raise UserWarning(u'Don\'t call list; it will not take advantage of prior prefetch optimizations')

    def all(self):
        return QuerySet(*self._elements)

    def distinct(self):

        values = \
            {
                tuple(
                    # remove the methods when comparing the uniqueness of the elements
                    [i for i in obj.__dict__.items() if i[0] not in ('save', 'delete')]
                ): obj for obj in self._elements
            }.values()
        return QuerySet(*values)

    def count(self):
        return len(self._elements)

    def exists(self):
        return len(self._elements)

    def filter(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], Q):
            q = args[0]
        else:
            q = Q(*args, **kwargs)
        queryset = QuerySet(*self._filter_q(q))
        queryset._selected = set(queryset._selected)
        queryset._prefetched = set(queryset._prefetched)
        return queryset

    def order_by(self, *attributes):
        def compare(x, y, attributes=attributes):
            for attr in attributes:
                if attr[:1] == '-':
                    attr = attr[1:]
                    if getattr(x, attr) > getattr(y, attr):
                        return -1
                    if getattr(x, attr) < getattr(y, attr):
                        return 1
                else:
                    if getattr(x, attr) < getattr(y, attr):
                        return -1
                    if getattr(x, attr) > getattr(y, attr):
                        return 1
            return 0
        elements = list(self._elements[:])
        elements.sort(compare)
        return QuerySet(*elements)

    def get(self, **kwargs):
        filtered_elements = list(self._filter_elements(**kwargs))
        count = len(filtered_elements)

        if count == 0:
            # This is not really correct, should return an exception that
            # matches the specific sub-class, but this is close enough for
            # the purpose of many tests.
            raise self.model.DoesNotExist
        elif count == 1:
            return filtered_elements[0]
        else:
            raise MultipleObjectsReturned

    def select_related(self, *fields):
        queryset = QuerySet(*self._elements)

        if len(fields) == 0:
            # Django selects all related when called with no arguments
            # That behavior is semi-pathological, so fake some unintended relations
            queryset._selected = {'some', 'unintended', 'relations'}
        elif len(fields) == 1 and fields[0] is None:
            # emulate the clearing behavior when passed None
            queryset._selected = set()
        else:
            # Accurately, recreate Django's broken behavior:
            # https://code.djangoproject.com/ticket/16855
            queryset._selected = set(fields)

        queryset._prefetched = set(self._prefetched)
        return queryset

    def prefetch_related(self, *fields):
        queryset = QuerySet(*self._elements)
        queryset._selected = set(self._selected)

        if len(fields) == 1 and fields[0] is None:
            # emulate the clearing behavior when passed None
            queryset._prefetched = set()
        else:
            queryset._prefetched = set(self._prefetched) | set(fields)

        return queryset

    def _filter_q(self, q):
        if not q.children:
            return set(self._elements)

        if q.connector == 'AND':
            opp = operator.and_
        else:
            opp = operator.or_

        results = None
        for child in q.children:
            if isinstance(child, Q):
                r = self._filter_q(child)
            else:
                r = self._filter_elements(**dict([child]))

            if results is None:
                results = r
            else:
                results = opp(results, r)

        return set(results)

    def _filter_elements(self, **kwargs):
        filtered_elements = self._elements
        for attr, value in kwargs.iteritems():
            if attr.endswith('__lt'):
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr[:-4]) < value]
            elif attr.endswith('__gt'):
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr[:-4]) > value]
            elif attr.endswith('__lte'):
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr[:-5]) <= value]
            elif attr.endswith('__gte'):
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr[:-5]) >= value]
            elif attr.endswith('__in'):
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr[:-4]) in value]
            else:
                filtered_elements =\
                    [element for element in self._elements if
                        getattr(element, attr) == value]
        return set(filtered_elements)


class Manager(Mock):
    def all(self):
        return QuerySet()

    def __iter__(self):
        return iter(self.all())


class FieldsInitType(type):
    def __new__(meta, classname, bases, dct):
        fields = dct.get('_fields', [])
        for base in bases:
            if getattr(base, '_meta', None):
                base_meta = base._meta
                base_meta.fields = fields
                base_meta.many_to_many = []
                dct['_meta'] = base_meta
                break
        return type.__new__(meta, classname, bases, dct)


class Model(object):
    class DoesNotExist(ObjectDoesNotExist):
        pass

    _models = []
    _meta = Mock()
    objects = Manager()
    __metaclass__ = FieldsInitType

    def __init__(self, **kwargs):
        self.pk = getattr(self, 'pk', None)
        self._models.append(self)

        for key, value in kwargs.iteritems():
            setattr(self, key, value)

        def save_side_effect():
            if self.pk is None:
                self.pk = random.randint(1000, 10000)

        post_init.send(sender=self.__class__, instance=self)

        self.save = Mock(name='save', side_effect=save_side_effect)
        self.delete = Mock(name='delete')



import unittest

import mock

from django.db import models

from savory_pie.formatters import JSONFormatter
from savory_pie.django.haystack_filter import HaystackFilter
from savory_pie.django.haystack_resources import HaystackSearchResource
from savory_pie.django.haystack_field import HaystackField, ResourceIndex
from savory_pie.tests.mock_context import mock_context
from test_filters import Params


class TestModel(models.Model):
    pass


class TestSearchResource(HaystackSearchResource):
    model_class = TestModel


class FakeResourceIndex(ResourceIndex):
    def __init__(self):
        pass


class ResourcetIndextTestCase(unittest.TestCase):

    def test_get_model(self):
        fake_resource = FakeResourceIndex()
        fake_resource.resource_class = mock.Mock(spec=['model_class'])
        fake_resource.resource_class.model_class = 'SomeClass'
        self.assertEqual(fake_resource.get_model(), 'SomeClass')

    @mock.patch('savory_pie.django.haystack_field.Related.prepare')
    def test_prefetch_related(self, prepare):
        resource = FakeResourceIndex()
        resource.resource_class = mock.Mock(spec=['prepare'])
        resource.prefetch_related = mock.Mock()
        resource._prefetch_related('qs')
        self.assertTrue(resource.prefetch_related.called)
        prepare.assert_called_with('qs')


class HaystackFilterTest(unittest.TestCase):

    @mock.patch('savory_pie.django.haystack_filter.SearchQuerySet')
    def test_simple_filter(self, haystack_qs_cls):
        result1 = mock.Mock(name='result1', pk=1)
        result2 = mock.Mock(name='result2', pk=2)

        haystack_qs = haystack_qs_cls.return_value
        haystack_qs.filter.return_value = haystack_qs
        haystack_qs.models.return_value = haystack_qs
        haystack_qs.__iter__ = lambda x: iter([result1, result2])

        queryset = mock.Mock(name='queryset')
        queryset.model = TestModel

        haystack_filter = HaystackFilter()
        haystack_filter.filter(
            mock_context(),
            Params({'q': 'foo'}),
            queryset,
        )

        queryset.assert_has_calls(
            [
                mock.call.filter(pk__in=[1, 2]),
            ],
            any_order=True,
        )

        haystack_qs.assert_has_calls(
            [
                mock.call.models(TestModel),
                mock.call.filter(content=u'foo'),
            ],
            any_order=True
        )

    @mock.patch('savory_pie.django.haystack_filter.SearchQuerySet')
    def test_multi_word_filter(self, haystack_qs_cls):
        haystack_qs = haystack_qs_cls.return_value
        haystack_qs.filter.return_value = haystack_qs
        haystack_qs.models.return_value = haystack_qs

        queryset = mock.Mock(name='queryset')
        queryset.model = TestModel

        haystack_filter = HaystackFilter()
        haystack_filter.filter(
            mock_context(),
            Params({'q': 'foo bar baz'}),
            queryset,
        )

        haystack_qs.assert_has_calls(
            [
                mock.call.filter(content=u'foo'),
                mock.call.filter(content=u'bar'),
                mock.call.filter(content=u'baz'),
                mock.call.models(TestModel),
            ],
            any_order=True
        )


class HaystackFieldTest(unittest.TestCase):

    def test_haystack_field(self):
        FooResource = mock.Mock()
        FooResource.return_value = frv = mock.Mock()
        frv.get.return_value = {'a': 'b'}
        api = HaystackField(
            formatter=JSONFormatter(),
            resource=FooResource)
        self.assertEqual(api.prepare(None), '{"a": "b"}')


class HaystackSearchResourceTest(unittest.TestCase):

    @mock.patch('savory_pie.django.haystack_resources.SearchQuerySet')
    def test_haystack_search_resource(self, haystack_qs_cls):
        n = 3
        haystack_qs_cls.return_value = haystack_qs = mock.Mock()
        haystack_qs.filter.return_value = haystack_qs
        haystack_qs.models.return_value = haystack_qs
        haystack_qs.count.return_value = n
        result = mock.Mock()
        result.get_stored_fields.return_value = {'api': '"api-as-string"'}
        haystack_qs.__iter__ = mock.Mock(return_value=(result for i in range(n)))

        sr = HaystackSearchResource()
        sr.model_class = TestModel
        ctx = mock.Mock(base_uri='foo')
        self.assertEqual(
            ''.join([x for x in sr.get(ctx, {'q': 'foo'})]),
            '{"meta":{"count":3},"objects":["api-as-string","api-as-string","api-as-string"]}')
        haystack_qs.models.assert_called_with(TestModel)
        haystack_qs.count.assert_called_with()

    @mock.patch('savory_pie.django.haystack_resources._hash_string')
    @mock.patch('savory_pie.django.haystack_resources.SearchQuerySet')
    def test_all_results(self, SearchQuerySet, _hash_string):
        ctx = mock.Mock(base_uri='foo')

        _hash_string.side_effect = ['First', 'Second']

        result1 = mock.Mock()
        result1.get_stored_fields.return_value = {'api': '{"json":1}'}

        result2 = mock.Mock()
        result2.get_stored_fields.return_value = {'api': '{"json":2}'}

        qs = SearchQuerySet.return_value
        qs.models.return_value = qs
        qs.filter.return_value = qs
        qs.__iter__ = mock.Mock(return_value=iter([result1, result2]))
        qs.count.return_value = 2

        resource = TestSearchResource()
        result = resource.get(ctx, {})

        self.assertEqual(
            ''.join(result),
            '{"meta":{"count":2},"objects":['
            '{"json":1,"$hash":"First"},'
            '{"json":2,"$hash":"Second"}]}'
        )

        _hash_string.assert_any_call('{"json":1}')
        _hash_string.assert_any_call('{"json":2}')

        self.assertTrue(ctx.streaming_response)
        qs.assert_has_call(
            mock.call.models(TestModel),
            any_order=True,
        )

    @mock.patch('savory_pie.django.haystack_resources.SearchQuerySet')
    def test_q_filter(self, SearchQuerySet):
        ctx = mock.Mock(base_uri='foo')

        qs = SearchQuerySet.return_value
        qs.models.return_value = qs
        qs.filter.return_value = qs
        qs.__iter__ = mock.Mock(return_value=iter([]))
        qs.count.return_value = 0

        resource = TestSearchResource()
        result = resource.get(ctx, {'q': 'foo bar'})

        self.assertEqual(
            ''.join(result),
            '{"meta":{"count":0},"objects":[]}'
        )

        qs.assert_has_call(
            mock.call.filter('foo'),
            mock.call.filter('bar'),
            any_order=True,
        )

    @mock.patch('savory_pie.django.haystack_resources.SearchQuerySet')
    def test_updated_filter(self, SearchQuerySet):
        ctx = mock.Mock(base_uri='foo')

        qs = SearchQuerySet.return_value
        qs.models.return_value = qs
        qs.filter.return_value = qs
        qs.__iter__ = mock.Mock(return_value=iter([]))
        qs.count.return_value = 0

        resource = TestSearchResource()
        result = resource.get(ctx, {'updatedSince': '2012-01-01'})

        self.assertEqual(
            ''.join(result),
            '{"meta":{"count":0},"objects":[]}'
        )

        qs.assert_has_call(
            mock.call.filter('2012-01-01'),
            any_order=True,
        )



import hashlib

from collections import OrderedDict
from .errors import MethodNotAllowedError, PreConditionError
from .resources import EmptyParams, _ParamsImpl

try:
    import cStringIO as StringIO
except ImportError:
    import StringIO


def get_sha1(ctx, dct):
    # exclude keys like '$hash' from the hash
    hash_dict = OrderedDict()
    for key in dct.keys():
        if not key.startswith('$'):
            # Do not hash the magic variables
            hash_dict[key] = dct[key]

    buf = StringIO.StringIO()
    ctx.formatter.write_to(hash_dict, buf)

    return _hash_string(buf.getvalue())


def process_get_request(ctx, resource, get_params):
    if 'GET' in resource.allowed_methods:
        return resource.get(ctx, _ParamsImpl(get_params))
    else:
        raise MethodNotAllowedError(method='GET')


def process_post_request(ctx, resource, data):
    if 'POST' in resource.allowed_methods:
        return resource.post(ctx, data)
    else:
        raise MethodNotAllowedError(method='POST')


def process_put_request(ctx, resource, data, expected_hash=None):
    if 'PUT' in resource.allowed_methods:
        previous_content_dict = resource.get(ctx, EmptyParams())
        content_dict = resource.put(ctx, data,)
        # validation errors take precedence over hash mismatch
        if expected_hash and expected_hash != get_sha1(ctx, previous_content_dict):
            raise PreConditionError()
        else:
            return content_dict
    else:
        raise MethodNotAllowedError(method='PUT')


def process_delete_request(ctx, resource):
    if 'DELETE' in resource.allowed_methods:
        resource.delete(ctx)
    else:
        raise MethodNotAllowedError(method='DELETE')


def _hash_string(value):
    sha = hashlib.sha1(value)
    return sha.hexdigest()



import collections
import datetime
import re
import savory_pie


class ValidationError(Exception):
    def __init__(self, resource, errors):
        self.resource = resource
        self.errors = errors


def validate(ctx, key, resource, source_dict):
    """
    Descend through a resource, including its fields and any related resources
    or submodels, looking for validation errors in any resources or models whose
    validators flag issues with content therein.

    Parameters:

        ``ctx``

        ``key``
            the current path fragment of the dictionary key which will be used to store
            any errors found in the returned dict -- in the initial call to validate,
            this should probably be the name of the class being validated e.g. "user"

        ``resource``
            the ModelResource instance whose data is to be validated

        ``source_dict``

    Returns:

        a dict mapping dotted keys (representing resources or fields) to
        validation errors

    """
    key = ctx.formatter.convert_to_public_property(key)
    error_dict = {}
    if source_dict and resource:
        if hasattr(resource, 'fields') and \
           isinstance(resource.fields, collections.Iterable):
            for field in resource.fields:
                if not hasattr(field, 'name'):
                    continue

                fieldname = ctx.formatter.convert_to_public_property(field.name)
                if fieldname in source_dict:
                    value = source_dict[fieldname]
                    if isinstance(value, list):
                        # RelatedManagerField validator might want to examine parent dict
                        class SourceDictList(list):
                            def __init__(self, source_dicts, parent_dict):
                                self[:] = source_dicts
                                self.parent_dict = parent_dict

                        value = SourceDictList(value, source_dict)

                    # ignore validation if value hasn't changed
                    if hasattr(resource, 'model'):
                        try:
                            orig_value = getattr(resource.model, field.name, None)
                            if orig_value == ctx.formatter.to_python_value(type(orig_value), value):
                                continue
                        except Exception:
                            pass

                    # attempt to validate field
                    if hasattr(field, 'validator') and field.validator:
                        if isinstance(field.validator, collections.Iterable):
                            for validator in field.validator:
                                validator.find_errors(error_dict, ctx, key, resource, field, value)
                        else:
                            field.validator.find_errors(error_dict, ctx, key, resource, field, value)
                    if hasattr(field, 'validate_resource'):
                        error_dict.update(field.validate_resource(ctx, key, resource, value))

        if hasattr(resource, 'validators') and \
           isinstance(resource.validators, collections.Iterable):
            for validator in resource.validators:
                validator.find_errors(error_dict, ctx, key, resource, source_dict)
    return error_dict


class BaseValidator(object):

    """
    Validators are used to determine that the values of model fields are acceptable
    according to programmatically specifiable criteria::

        class ValidationTestResource(resources.ModelResource):
            parent_resource_path = 'users'
            model_class = User

            validators = [
                DatetimeFieldSequenceValidator('start_date', 'end_date')
            ]

            fields = [
                fields.AttributeField(attribute='name', type=str,
                    validator=StringFieldExactMatchValidator('Bob')),
                fields.AttributeField(attribute='age', type=int,
                    validator=(IntFieldMinValidator(21, 'too young to drink'),
                               IntFieldPrimeValidator(100))),
                    # A field can take either a single validator,
                    # or a list or tuple of multiple validators.
                fields.AttributeField(attribute='before', type=datetime),
                fields.AttributeField(attribute='after', type=datetime),
                fields.AttributeField(attribute='systolic_bp', type=int,
                    validator=IntFieldRangeValidator(100, 120,
                        'blood pressure out of range')),
            ]


    When you apply *BaseValidator.validate* to an instance of ValidationTestResource,
    it will check to see if all the criteria are satisfied, and will return a dict giving
    all violations as key-value pairs, where the keys are dotted Python names for the
    model or field in question, and the values are lists of error messages. So if several
    criteria fail to be met, you might see something like this::

        {
            'savory_pie.tests.django.test_validators.ValidationTestResource':
                ['Datetimes are not in expected sequence.'],
            'savory_pie.tests.django.test_validators.ValidationTestResource.age':
                ['too young to drink',
                 'This should be a prime number.'],
            'savory_pie.tests.django.test_validators.ValidationTestResource.name':
                ['This should exactly match the expected value.'],
            'savory_pie.tests.django.test_validators.ValidationTestResource.systolic_bp':
                ['blood pressure out of range']
        }

    You can write your own validators, like *IntFieldPrimeValidator* above::

        class IntFieldPrimeValidator(FieldValidator):

            error_message = 'This should be a prime number.'

            def __init__(self, maxprime):
                self._primes = _primes = [2, 3, 5, 7]
                def test_prime(x, _primes=_primes):
                    for p in _primes:
                        if p * p > x:
                            return True
                        if (x %% p) == 0:
                            return False
                for x in range(11, maxprime + 1, 2):
                    if test_prime(x):
                        _primes.append(x)

            def check_value(self, value):
                return value in self._primes

    As a general rule, a validator has a *find_errors* method which makes calls to the
    *check_value* method, and if errors are found, they are stored in a dict, keyed by
    the dotted name of the non-compliant model or field.

    """

    error_message = 'Validation failure message goes here'
    """
    The error message should give a clear description of the nature of the validation
    failure, if one occurs.
    """

    json_name = 'What the front end calls this validator'
    """
    This should be a name understood by the front-end developers as referring to this
    particular validator so that they can wire up JavaScript to validate HTML forms in
    the browser.
    """

    null = False
    """
    Ignore null values for any fields which should be validated.
    """

    def __init__(self, *args, **kwargs):
        self.error_message = kwargs.pop('error_message', self.error_message)
        self.null = kwargs.pop('null', False)
        self.populate_schema(**kwargs)

    def _add_error(self, error_dict, key, error):
        if key in error_dict:
            error_dict[key].append(error)
        else:
            error_dict[key] = [error]

    def populate_schema(self, **kwargs):
        """
        Every validator *MUST* call this method in its constructor. The *kwargs*
        should be name-value pairs for any parameters required for validation. If the
        constructor sets error_message, that should happen *before* the call to this
        method.
        """
        self._schema = schema = {
            'name': self.json_name,
            'text': self.error_message
        }
        for key, value in kwargs.items():
            schema[key] = value

    def to_schema(self):
        """
        Subclasses are expected to overload this method with a string used in
        the front end for HTML form validation, for example in the context of
        something like `jQuery-Validation-Engine`_.

        .. _`jQuery-Validation-Engine`: https://github.com/posabsolute/jQuery-Validation-Engine

        Returns:

            a string representing the constraints on this resource or field, in a form
            that's useful on the front end, e.g. JavaScript
        """
        return self._schema

    def check_value(self, value):
        """
        Extend this method to test whatever needs testing on a model or field. Return
        True if the value is OK, False if it's unacceptable.
        """
        return False


########## Resource validators ###########


class ResourceValidator(BaseValidator):

    """
    Base class for validators that apply to ModelResource instances. These will usually
    look at relationships between field values, as the fields themselves will be
    individually validated.
    """

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        """
        Search for validation errors in the database model underlying a resource.
        """
        if not self.check_value(source_dict):
            self._add_error(error_dict, key, self.error_message)


class DatetimeFieldSequenceValidator(ResourceValidator):

    """
    Test an AttributeField of type 'int' to make sure it falls within a given
    range (inclusive at both ends).

    Parameters:

        ``*date_fields``
            a list of names of AttributeFields of type 'datetime' which are
            required to be in chronological sequence

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'dates_in_sequence'

    error_message = 'Datetimes are not in expected sequence.'

    def __init__(self, *args, **kwargs):
        kwargs['fields'] = ','.join(args)
        super(DatetimeFieldSequenceValidator, self).__init__(**kwargs)
        self._date_fields = args

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        """
        Verify that specified datetime fields exist, and are in chronological sequence
        as expected.
        """
        values = []
        for attr in self._date_fields:
            public_attr = ctx.formatter.convert_to_public_property(attr)
            if self.null and source_dict.get(public_attr) is None:
                return
            elif public_attr not in source_dict:
                self._add_error(error_dict, key,
                                'Cannot find datetime field "' + attr + '"')
                return
            values.append(ctx.formatter.to_python_value(datetime.datetime,
                                                        source_dict[public_attr]))
        for before, after in zip(values[:-1], values[1:]):
            if before > after:
                self._add_error(error_dict, key, self.error_message)
                return


class RequiredFieldValidator(ResourceValidator):
    """
    Ensure that a particular field of a resource exists.

    Parameters:

        ``field``
            the name of the required field

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'required_field'

    error_message = 'This field is required'

    def __init__(self, field, *args, **kwargs):
        self.field = field
        super(RequiredFieldValidator, self).__init__(**kwargs)

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        json_key = ctx.formatter.convert_to_public_property(self.field)
        if not source_dict.get(json_key):
            self._add_error(error_dict, key, self.error_message + ': ' + json_key)


class RequiredTogetherValidator(ResourceValidator):
    """
    Test a tuple of fields to ensure that if a value for one field in the set is
    provided, that all fields in the set have values provided.

    Parameters:

        ``*fields``
            a list of names of savory_pie Fields, which as a set is required if a
            value for any single field is provided

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'required_together'

    error_message = 'This set of fields is required if any one is provided.'

    def __init__(self, *args, **kwargs):
        kwargs['fields'] = ','.join(args)
        super(RequiredTogetherValidator, self).__init__(**kwargs)
        self._fields = args

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        values = {}
        for attr in self._fields:
            public_attr = ctx.formatter.convert_to_public_property(attr)
            if not self.null and public_attr not in source_dict:
                self._add_error(error_dict, key, 'Cannot find field "' + attr + '"')
                return
            values[attr] = source_dict.get(public_attr)

        missing = False
        required = False
        for value in values.values():
            if value is None:
                missing = True
            else:
                required = True
        if required and missing:
            self._add_error(error_dict, key, self.error_message)


class UniqueTogetherValidator(ResourceValidator):
    """
    Test a tuple of fields to ensure their proposed values represent a unique set
    within the database. This validator is similar to Django ORM's 'unique together'
    constraint, but differs in that it accepts only a single level of fields:
        https://docs.djangoproject.com/en/dev/ref/models/options/#unique-together

    Parameters:

        ``*fields``
            a list of names of savory_pie Fields, which as a set should be unique

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'unique_together'

    error_message = 'This set of fields must be unique.'

    def __init__(self, *args, **kwargs):
        kwargs['fields'] = ','.join(args)
        super(UniqueTogetherValidator, self).__init__(**kwargs)
        self._fields = args

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        filters = []
        for attr in self._fields:
            public_attr = ctx.formatter.convert_to_public_property(attr)
            if self.null and source_dict.get(public_attr) is None:
                return
            elif public_attr not in source_dict:
                self._add_error(error_dict, key, 'Cannot find field "' + attr + '"')
                return

            for field in resource.fields:
                if attr == getattr(field, 'name', None):
                    try:
                        if field.__class__ == savory_pie.django.fields.SubModelResourceField:
                            if 'resourceUri' in source_dict[public_attr]:
                                pk = source_dict[public_attr]['resourceUri'].split('/')[-1]
                                filters.append({'{}__pk'.format(attr): pk})
                            elif 'name' in source_dict[public_attr]:
                                filters.append({'{}__name'.format(attr): source_dict[public_attr]['name']})
                            else:
                                #TODO allow lookup by fields other than id/name?
                                return
                        elif issubclass(field.__class__, savory_pie.django.fields.AttributeField):
                            filters.append({attr: source_dict[public_attr]})
                    except Exception:
                        pass

        if filters and hasattr(resource, 'model'):
            try:
                qset = resource.model.__class__.objects.all()
                for f in filters:
                    qset = qset.filter(**f)
                if len(qset):
                    # if validation fails because we're re-saving an existing object, ignore
                    if len(qset) == 1 and resource.model.pk and qset[0].pk == resource.model.pk:
                        return
                    self._add_error(error_dict, key, self.error_message)
            except Exception:
                pass


class UniquePairedFieldValidator(ResourceValidator):
    """
    Test a pair of fields (a, b), such that for a given a, only one b can exist. However,
    this _unique_ combination of fields can exist unlimited times.

    Parameters:

        ``*fields``
            a pair of savory_pie Fields (a, b), such that for a given a,
            only one b can exist

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'unique_paired_field'

    error_message = 'First field is already present in another pair.'

    def __init__(self, *args, **kwargs):
        kwargs['fields'] = ','.join(args)
        super(UniquePairedFieldValidator, self).__init__(**kwargs)
        self._fields = args

    def find_errors(self, error_dict, ctx, key, resource, source_dict):
        filters = []
        for attr in self._fields:
            public_attr = ctx.formatter.convert_to_public_property(attr)
            if self.null and source_dict.get(public_attr) is None:
                return
            elif public_attr not in source_dict:
                self._add_error(error_dict, key, 'Cannot find field "' + attr + '"')
                return

            for field in resource.fields:
                if attr == getattr(field, 'name', None):
                    try:
                        if field.__class__ == savory_pie.django.fields.SubModelResourceField:
                            if 'resourceUri' in source_dict[public_attr]:
                                pk = source_dict[public_attr]['resourceUri'].split('/')[-1]
                                filters.append({'{}__pk'.format(attr): pk})
                            elif 'name' in source_dict[public_attr]:
                                filters.append({'{}__name'.format(attr): source_dict[public_attr]['name']})
                            else:
                                #TODO allow lookup by fields other than id/name?
                                return
                        elif issubclass(field.__class__, savory_pie.django.fields.AttributeField):
                            filters.append({attr: source_dict[public_attr]})
                    except Exception:
                        pass

        if filters and hasattr(resource, 'model'):
            try:
                qset = resource.model.__class__.objects.filter(**filters[0]).exclude(**filters[1])
                if len(qset):
                    self._add_error(error_dict, key, self.error_message)
            except Exception:
                pass


########## Field validators ############


class FieldValidator(BaseValidator):

    """
    Base class for all validators of fields: AttributeField, URIResourceField,
    SubObjectResourceField, IterableField
    """

    def find_errors(self, error_dict, ctx, key, resource, field, value):
        """
        Search for validation errors in a field of a database model.
        """
        fieldname = ctx.formatter.convert_to_public_property(field.name)
        value = ctx.formatter.to_python_value(field._type, value)
        if value is None:
            if self.null:
                return
            self._add_error(error_dict, key + '.' + fieldname, '{} is required'.format(fieldname))
        if not self.check_value(value):
            self._add_error(error_dict, key + '.' + fieldname, self.error_message)


class StringFieldZipcodeValidator(FieldValidator):

    """
    Test an AttributeField of type 'str' to make sure it's a valid zipcode.

    **TODO**:

        Handle international postal codes, some are six digits???
    """

    json_name = 'us_zipcode'

    error_message = 'This should be a zipcode.'

    pattern = re.compile(r'^\d{5}(-\d{4})?$')

    def check_value(self, value):
        """
        Verify that the value is a five-digit string.

        """
        try:
            return self.pattern.match(value)
        except TypeError:
            return False


class StringFieldExactMatchValidator(FieldValidator):

    """
    Test an AttributeField of type 'str' to make sure it exactly matches an
    expected value.

    Parameters:

        ``expected``
            the case-sensitive string value that we expect to see

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'exact_string'

    error_message = 'This should exactly match the expected value.'

    def __init__(self, expected, **kwargs):
        kwargs['expected'] = expected
        super(StringFieldExactMatchValidator, self).__init__(**kwargs)
        self._expected = expected

    def check_value(self, value):
        """
        Verify that the value is a string exactly matching the constructor argument.
        """
        return value == self._expected


class StringFieldMaxLengthValidator(FieldValidator):

    """
    Test an AttributeField of type 'str' to make sure it does not exceed the
    expected length.

    Parameters:

        ``expected_length``
            the maximum length for an allowable string

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'string_maxlen'

    error_message = 'This should not exceed the expected string length.'

    def __init__(self, expected_length, **kwargs):
        kwargs['expected_length'] = expected_length
        super(StringFieldMaxLengthValidator, self).__init__(**kwargs)
        self._expected_length = expected_length

    def check_value(self, value):
        """
        Verify that the value is a string whose length doesn't exceed the maximum.
        """
        return ((isinstance(value, str) or isinstance(value, unicode))
                and len(value) <= self._expected_length)


class IntFieldMinValidator(FieldValidator):

    """
    Test an AttributeField of type 'int' to make sure it is no smaller than a
    specified minimum.

    Parameters:

        ``min``
            the specified minimum

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'int_min'

    error_message = 'This value should be greater than or equal to the minimum.'

    def __init__(self, _min, **kwargs):
        kwargs['min'] = _min
        super(IntFieldMinValidator, self).__init__(**kwargs)
        self._min = _min

    def check_value(self, intvalue):
        """
        Verify integer value is no less than specified minimum.
        """
        return type(intvalue) is int and intvalue >= self._min


class IntFieldMaxValidator(FieldValidator):

    """
    Test an AttributeField of type 'int' to make sure it is no greater than a
    specified maximum.

    Parameters:

        ``max``
            the specified maximum

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'int_max'

    error_message = 'This value should be less than or equal to the maximum.'

    def __init__(self, _max, **kwargs):
        kwargs['max'] = _max
        super(IntFieldMaxValidator, self).__init__(**kwargs)
        self._max = _max

    def check_value(self, intvalue):
        """
        Verify integer value is no greater than specified maximum.
        """
        return type(intvalue) is int and intvalue <= self._max


class IntFieldRangeValidator(FieldValidator):

    """
    Test an AttributeField of type 'int' to make sure it falls within a given
    range (inclusive at both ends).

    Parameters:

        ``min``
            the bottom of the range

        ``max``
            the top of the range

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'int_range'

    error_message = 'This value should be within the allowed integer range.'

    def __init__(self, _min, _max, **kwargs):
        kwargs.update({'min': _min, 'max': _max})
        super(IntFieldRangeValidator, self).__init__(**kwargs)
        self._min, self._max = _min, _max

    def check_value(self, intvalue):
        """
        Verify that numerical value is within specified range.
        """
        return type(intvalue) is int and intvalue >= self._min and intvalue <= self._max


class DatetimeFieldMinValidator(FieldValidator):

    """
    Test an AttributeField of type datetime to make sure it is no earlier than a
    specified minimum.

    Parameters:

        ``min``
            the specified minimum

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'datetime_min'

    error_message = 'This value should be no earlier than the minimum datetime.'

    def __init__(self, _min, **kwargs):
        kwargs['min'] = _min.isoformat()
        super(DatetimeFieldMinValidator, self).__init__(**kwargs)
        self._min = _min

    def check_value(self, datetimevalue):
        """
        Verify integer value is no less than specified minimum.
        """
        return type(datetimevalue) is datetime.datetime and datetimevalue >= self._min


class DatetimeFieldMaxValidator(FieldValidator):

    """
    Test an AttributeField of type datetime to make sure it is no later than a
    specified maximum.

    Parameters:

        ``max``
            the specified maximum

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'datetime_max'

    error_message = 'This value should be no later than the maximum datetime.'

    def __init__(self, _max, **kwargs):
        kwargs['max'] = _max.isoformat()
        super(DatetimeFieldMaxValidator, self).__init__(**kwargs)
        self._max = _max

    def check_value(self, datetimevalue):
        """
        Verify integer value is no greater than specified maximum.
        """
        return type(datetimevalue) is datetime.datetime and datetimevalue <= self._max


class DateFieldMinValidator(FieldValidator):

    """
    Test an AttributeField of type date to make sure it is no earlier than a
    specified minimum.

    Parameters:

        ``min``
            the specified minimum

        ``error_message``
            optional: the message to appear in the error dictionary if this
            condition is not met
    """

    json_name = 'date_min'

    error_message = 'This value should be no earlier than the minimum date.'

    def __init__(self, _min, **kwargs):
        kwargs['min'] = _min.isoformat()
        super(DateFieldMinValidator, self).__init__(**kwargs)
        self._min = _min

    def check_value(self, datevalue):
        """
        Verify integer value is no less than specified minimum.
        """
        return type(datevalue) is datetime.date and datevalue >= self._min



import unittest
from mock import Mock

import django.core.exceptions

from savory_pie.django import resources, fields
from savory_pie.tests.django import mock_orm
from savory_pie.tests.mock_context import mock_context


def createCreator(model_class, collection):
    def createObject(**kwargs):
        obj = model_class(**kwargs)
        collection.append(obj)
        return obj
    return createObject

####################

_groups = []


class Group(mock_orm.Model):
    name = Mock(name='group.name')
    pk = Mock(name='group.pk')


class GroupResource(resources.ModelResource):
    parent_resource_path = 'groups'
    model_class = Group

    fields = [
        fields.AttributeField('name', type=str),
    ]


class GroupQuerySetResource(resources.QuerySetResource):
    resource_path = 'group'
    resource_class = GroupResource

createGroup = createCreator(Group, _groups)

######################

_people = []


class Person(mock_orm.Model):
    name = Mock(name='person.name')
    groups = Mock(name='person.groups')
    pk = Mock(name='person.pk')

Person.groups.all.return_value = _groups
del Person.groups.add      # this is a "through" relationship
del Person.groups.remove
Person.groups.source_field_name = 'person'
Person.groups.target_field_name = 'group'


class PersonResource(resources.ModelResource):
    parent_resource_path = 'person'
    model_class = Person

    fields = [
        fields.AttributeField('name', type=str),
        fields.RelatedManagerField('groups', GroupResource),
    ]


class PersonQuerySetResource(resources.QuerySetResource):
    resource_path = 'person'
    resource_class = PersonResource

createPerson = createCreator(Person, _people)

######################

_memberships = []


class Membership(mock_orm.Model):
    group = Mock(django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor(Mock()))
    person = Mock(django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor(Mock()))
    pk = Mock()
    objects = Mock()


class MembershipResource(resources.ModelResource):
    parent_resource_path = 'membership'
    model_class = Membership

    fields = [
        fields.RelatedManagerField('group', GroupResource),
        fields.RelatedManagerField('person', PersonResource),
    ]

Person.groups.through = Membership


class MembershipQuerySetResource(resources.QuerySetResource):
    resource_path = 'membership'
    resource_class = MembershipResource

createMembership = createCreator(Membership, _memberships)

Membership.objects.create = createMembership

##########


class ManyToManyThroughTest(unittest.TestCase):

    def setUp(self):
        createGroup(name='Rotary Club', pk=1)
        createGroup(name='Boston Python Meetup', pk=2)
        createPerson(name='Alice', pk=3)
        createPerson(name='Bob', pk=4)
        # Alice is a member of the Rotary Club
        createMembership(group=_groups[0], person=_people[0], pk=5)
        # Bob is a member of the Boston Python Meetup
        createMembership(group=_groups[1], person=_people[1], pk=6)

    def tearDown(self):
        global _groups, _people, _memberships
        del _groups[:]
        del _people[:]
        del _memberships[:]

    def test_check_add(self):
        related = PersonResource.fields[1]
        # When the add method is missing, we know it's a "through" M2M relationship.
        self.assertFalse(hasattr(related, 'add'))

    @unittest.skipIf(
        True,
        """
        This test is broken, but somewhat replaced by RelatedManagerFieldTest.test_incoming_m2m_add
        and RelatedManagerFieldTest.test_incoming_m2m_delete'
        """
    )
    def test_m2m_through(self):
        ctx = mock_context()

        def resolve(*args):
            prefix = 'http://localhost:8000/api/'
            self.assertTrue(args[0].startswith(prefix))
            arg = args[0][len(prefix):]
            if arg.startswith('group/'):
                n = int(arg[6:]) - 1
                assert n < len(_groups), n
                return GroupResource(_groups[n])
            elif arg.startswith('person/'):
                n = int(arg[7:]) - 1
                assert n < len(_people), n
                return PersonResource(_people[n])
            else:
                self.fail(arg)

        ctx.resolve_resource_uri = resolve
        source_dict = {
            'groups': [{'resourceUri': 'http://localhost:8000/api/group/1', 'name': 'Boy Scouts'}],
            'resourceUri': 'http://localhost:8000/api/person/1',
            'name': 'Charlie'
        }
        resource = PersonResource(_people[0])
        resource.put(ctx, source_dict)
        self.assertEqual('Boy Scouts', _groups[0].name)
        self.assertEqual(3, len(_memberships))
        resource = GroupResource(_groups[0])
        self.assertEqual({'resourceUri': 'uri://groups/1',
                          'name': 'Boy Scouts',
                          '$hash': 'a35a8e769bb1583a840525d1e8fd6b3d02658b04'},
                         resource.get(ctx, {'resourceUri': 'http://localhost:8000/api/group/1'}))



class DjangoUserPermissionValidator(object):
    """
    Permissions Validator is used to tie into an authorization.  Is used in conjunction with the authorization decorator
    Added to the field init method.
    """
    def __init__(self, permission_name, auth_adapter=None):
        self.permission_name = permission_name
        self.auth_adapter = auth_adapter

    def is_write_authorized(self, ctx, target_obj, source, target):
        """
        Leverages the users has_perm(key) method to leverage the authorization.
        Only check if the source and target have changed.
        """
        user = ctx.request.user

        if source != target:
            return user.has_perm(self.permission_name)

        return True

    def fill_schema(self, schema_dict):
        # TODO: implement fill_schema
        pass



from collections import OrderedDict
from datetime import datetime
from mock import Mock, MagicMock, call, patch
import json
import unittest

from django.contrib.auth.models import User as DjangoUser
from django.http import QueryDict
from savory_pie.django import resources, fields, views
from savory_pie.django.filters import ParameterizedFilter
from savory_pie.tests.django import user_resource_schema, mock_orm, date_str
from savory_pie.tests.mock_context import mock_context
from savory_pie.resources import EmptyParams, _ParamsImpl
from savory_pie.errors import SavoryPieError
from savory_pie import formatters
import django.core.exceptions


class ResourceTest(unittest.TestCase):
    def test_no_allowed_methods(self):
        resource = resources.Resource()
        self.assertEqual(resource.allowed_methods, set())

    def test_allows_get(self):
        resource = resources.Resource()
        resource.get = lambda: dict()
        self.assertEqual(resource.allowed_methods, {'GET'})


class User(mock_orm.Model):
    name = Mock()
    age = Mock()


class UserOwner(mock_orm.Model):
    user = Mock(django.db.models.fields.related.ReverseSingleRelatedObjectDescriptor(Mock()))
    name = Mock()


class AddressableUserResource(resources.ModelResource):
    parent_resource_path = 'users'
    model_class = User

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int)
    ]


class UnaddressableUserResource(resources.ModelResource):
    model_class = User

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int)
    ]


class SemiUnaddressableUserResource(UnaddressableUserResource):
    parent_resource_path = 'users'


class ComplexUserResource(resources.ModelResource):
    model_class = User

    fields = [
        fields.SubModelResourceField(attribute='manager', resource_class=UnaddressableUserResource),
        fields.RelatedManagerField(attribute='reports', resource_class=UnaddressableUserResource)
    ]


class UserOwnerResource(resources.ModelResource):
    model_class = UserOwner

    fields = [
        fields.AttributeField(attribute='name', type=str),
    ]


class ComplexUserRelationResource(resources.ModelResource):
    model_class = User

    owner_field = fields.SubModelResourceField(attribute='owner', resource_class=UserOwnerResource)

    fields = [
        fields.AttributeField(attribute='name', type=str),
        fields.AttributeField(attribute='age', type=int),
        owner_field,
    ]


class ModelResourceTest(unittest.TestCase):
    def make_request(self, _json, sha=None):
        request = Mock()
        request.META = Mock()
        request.META.get.return_value = sha
        request.read.return_value = _json
        return request

    def test_resource_path(self):
        user = User(pk=1, name='Bob', age=20)
        resource = AddressableUserResource(user)

        self.assertEqual(resource.resource_path, 'users/1')

    @patch('savory_pie.django.resources.dirty_bits')
    def test_dirty_signal_register(self, dirty_bits):
        class NewClazz(mock_orm.Model):
            pass

        class NewClazzResource(resources.ModelResource):
            model_class = NewClazz

        NewClazzResource(NewClazz())
        dirty_bits.register.assert_called_with(NewClazz)

    def test_resource_get_returns_hash(self):
        user = User(pk=1, name='Bob', age=20)

        resource = AddressableUserResource(user)
        dct = resource.get(mock_context(), EmptyParams())

        self.assertEqual(dct, {
            'name': 'Bob',
            'age': 20,
            'resourceUri': 'uri://users/1'
        })

    def test_put(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20
        })

        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertTrue(user.save.called)

    def test_complex_put(self):
        user = User()
        user._meta.get_field().related.field.name = 'name'
        user.owner = UserOwner()

        resource = ComplexUserRelationResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20,
            'owner': {'name': 'bob owner'},
        })

        self.assertTrue(user.save.called)
        self.assertTrue(user.owner.save.called)
        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertEqual(user.owner.name, 'bob owner')

    def test_clean_save(self):

        age_field = Mock()
        age_field.name = 'age'
        age_field.value_to_string.return_value = 30

        name_field = Mock()
        name_field.name = 'name'
        name_field.value_to_string.return_value = 'Bob'

        class DirtyUser(mock_orm.Model):
            _fields = [age_field, name_field]
            pk = 3

        class DirtyUserResource(resources.ModelResource):
            parent_resource_path = 'users'
            model_class = DirtyUser

            fields = [
                fields.AttributeField(attribute='name', type=str),
                fields.AttributeField(attribute='age', type=int)
            ]

        dirty_user = DirtyUser()
        dirty_user.save = Mock()
        resource = DirtyUserResource(dirty_user)

        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 30,
            'resourceUri': 'uri://users/1'
        })
        self.assertFalse(dirty_user.save.called)

    def test_dirty_save(self):
        age_field = Mock()
        age_field.name = 'age'
        age_field.value_to_string.return_value = 30

        name_field = Mock()
        name_field.name = 'name'
        name_field.value_to_string.return_value = 'Bob'

        class DirtyUser(mock_orm.Model):
            _fields = [age_field, name_field]
            pk = 3

        class DirtyUserResource(resources.ModelResource):
            parent_resource_path = 'users'
            model_class = DirtyUser

            fields = [
                fields.AttributeField(attribute='name', type=str),
                fields.AttributeField(attribute='age', type=int)
            ]

        dirty_user = DirtyUser()
        dirty_user.save = Mock()
        dirty_user.is_dirty = lambda: True
        resource = DirtyUserResource(dirty_user)

        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 30,
            'resourceUri': 'uri://users/1'
        })
        self.assertTrue(dirty_user.save.called)

    def _dict_compare(self, actual, expected):
        for item1, item2 in zip(expected.items(), actual.items()):
            self.assertEqual(item1, item2, 'Actual not equal to expected {0} {1}'.format(actual, expected))

    def test_qsr_returns_hashes(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 2
        })

        result = data['objects']
        self.assertEqual(len(result), 2)
        first_result = OrderedDict()
        second_result = OrderedDict()

        for key in sorted(result[0].keys()):
            first_result[key] = result[0][key]

        for key in sorted(result[1].keys()):
            second_result[key] = result[1][key]

        alice = OrderedDict({
            '$hash': '01a1b638ddf5318259419587f95ca091a179eeb5',
            'age': 31,
            'name': 'Alice',
            'resourceUri': 'uri://users/1', })
        bob = OrderedDict({
            '$hash': 'df8c8b5694bcd438ea86a87414cf3f59ca42a051',
            'age': 20,
            'name': 'Bob',
            'resourceUri': 'uri://users/2', })

        if first_result['name'] == 'Alice':
            self._dict_compare(first_result, alice)
            self._dict_compare(second_result, bob)
        else:
            self._dict_compare(second_result, alice)
            self._dict_compare(first_result, bob)

    def test_put_with_good_sha(self):
        user = User()
        resource = AddressableUserResource(user)
        resource.get = lambda *args: {"age": 20, "name": "Bob"}
        request = self.make_request('{"age": 20, "name": "Bob"}', 'fd92376f24d6a75974c8da6edf84a834b92ee13c')
        response = views._process_put(mock_context(), resource, request)
        self.assertEqual(response.status_code, 204)

    def test_put_with_bad_sha(self):
        user = User()
        resource = AddressableUserResource(user)
        resource.get = lambda *args: {"age": 20, "name": "Bob"}
        request = self.make_request('{"age": 20, "name": "Bob"}', 'OmManePadmeHumOmManePadmeHumOmManePadmeHum')
        response = views._process_put(mock_context(), resource, request)
        self.assertEqual(response.status_code, 412)

    def test_put_with_missing_required_field(self):
        user = User()
        request = self.make_request('{"name": "Bob"}')   # no age
        response = views._process_put(mock_context(),
                                      AddressableUserResource(user),
                                      request)
        errors = json.loads(response.content)
        self.assertTrue('validation_errors' in errors)
        self.assertEqual(errors['validation_errors'], {'missingField': 'age', 'target': 'User'})

    def test_put_with_foreign_key_none_resource(self):
        user = User()

        resource = ComplexUserRelationResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20,
            'owner': {'name': 'bob owner'},
        })

        self.assertTrue(user.save.called)
        self.assertTrue(user.owner.save.called)
        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertEqual(user.owner.name, 'bob owner')

    def test_put_with_save_false(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.put(mock_context(), {
            'name': 'Bob',
            'age': 20
        }, save=False)

        self.assertEqual(user.name, 'Bob')
        self.assertEqual(user.age, 20)
        self.assertFalse(user.save.called)

    def test_delete(self):
        user = User()

        resource = AddressableUserResource(user)
        resource.delete(mock_context())

        self.assertTrue(user.delete.called)

    def test_get_by_source_dict(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(AddressableUserResource.model_class, 'objects') as objects:
            AddressableUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with(name='Bob', age=15)

    def test_get_by_source_dict_not_found(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(AddressableUserResource.model_class, 'objects') as objects:
            objects.filter.get.return_value = django.core.exceptions.ObjectDoesNotExist
            AddressableUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with(name='Bob', age=15)

    def test_get_by_source_dict_filter_by_item_optional(self):
        source_dict = {
            'name': 'Bob',
            'age': 15,
        }

        with patch.object(ComplexUserResource.model_class, 'objects') as objects:
            ComplexUserResource.get_by_source_dict(mock_context(), source_dict)

        objects.filter.assert_called_with()

    def test_pre_save_optional(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        resource.put(mock_context(), {'foo': 'bar'})

        user.save.assert_called_with()

    def test_save_fields(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming', 'save'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        resource.put(mock_context(), {'foo': 'bar'})

        self.assertTrue(field.save.called)

    def test_save_fields_optional(self):
        user = Mock(name='user')
        field = Mock(['handle_incoming'])

        class Resource(resources.ModelResource):
            model_class = User
            fields = [field]
        resource = Resource(user)

        # Tests that an attribute error is not raised
        resource.put(mock_context(), {'foo': 'bar'})


class AddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = AddressableUserResource


class SemiUnaddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = SemiUnaddressableUserResource


class FullyUnaddressableUserQuerySetResource(resources.QuerySetResource):
    resource_class = UnaddressableUserResource


class ComplexUserResourceQuerySetResource(resources.QuerySetResource):
    resource_class = ComplexUserResource


class QuerySetResourceTest(unittest.TestCase):
    def remove_hash(self, dct):
        return dict((k, v) for k, v in dct.items() if k[:1] != '$')

    def test_query_set_get(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 2
        })

        self.assertEqual(len(data['objects']), 2)

        data = sorted(data['objects'], key=lambda k: k['name'])

        self.assertEqual(map(self.remove_hash, data), [
            {'resourceUri': 'uri://users/1', 'name': 'Alice', 'age': 31},
            {'resourceUri': 'uri://users/2', 'name': 'Bob', 'age': 20}
        ])

    def test_query_set_get_disallow_unfiltered_query(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        data = None
        with self.assertRaises(SavoryPieError):
            data = resource.get(mock_context(), EmptyParams())
        self.assertEqual(data, None)  # we should not ever have any data as this is not allowed

    def test_query_set_get_with_valid_filter_param(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        resource.filters = [ParameterizedFilter('name', 'name')]
        data = resource.get(mock_context(), _ParamsImpl(QueryDict('name=Alice')))
        self.assertEqual(data['objects'][0]['name'], 'Alice')

    def test_query_set_get_with_invalid_filter_param(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))
        resource.allow_unfiltered_query = False
        resource.filters = [ParameterizedFilter('name', 'name')]
        data = None
        with self.assertRaises(SavoryPieError):
            data = resource.get(mock_context(), _ParamsImpl(QueryDict('[object Object]=12345')))
        self.assertEqual(data, None)

    def test_get_distinct(self):
        resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=1, name='Alice', age=31)
        ))
        data = resource.get(mock_context(), EmptyParams())
        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 1
        })

        self.assertEqual(map(self.remove_hash, data['objects']), [
            {'resourceUri': 'uri://users/1', 'name': 'Alice', 'age': 31},
        ])

    def test_empty_queryset(self):
        mock_query_set = MagicMock()
        mock_query_set.__bool__ = False
        mock_query_set.return_value = None
        resource = AddressableUserQuerySetResource(mock_query_set)
        data = resource.get(mock_context(), EmptyParams())

        self.assertEqual(data['meta'], {
            'resourceUri': 'uri://users',
            'count': 0
        })

        self.assertEqual(data['objects'], [])

    def test_addressable_post(self):
        queryset_resource = AddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })

        new_user = new_resource.model
        self.assertEqual(new_resource.resource_path, 'users/' + str(new_user.pk))

        self.assertEqual(new_user.name, 'Bob')
        self.assertEqual(new_user.age, 20)
        self.assertTrue(new_user.save.called)

    def test_semi_unaddressable_post(self):
        queryset_resource = SemiUnaddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })
        self.assertEqual(new_resource.resource_path, 'users/' + str(new_resource.model.pk))

    def test_fully_unaddressable_post(self):
        queryset_resource = FullyUnaddressableUserQuerySetResource()

        new_resource = queryset_resource.post(mock_context(), {
            'name': 'Bob',
            'age': 20
        })
        self.assertIsNone(new_resource.resource_path)

    def test_get_child_resource_success(self):
        alice = User(pk=1, name='Alice', age=31)
        bob = User(pk=2, name='Bob', age=20)

        queryset_resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            alice,
            bob
        ))

        model_resource = queryset_resource.get_child_resource(mock_context(), 1)
        self.assertEqual(model_resource.model, alice)

    def test_get_child_resource_fail(self):
        queryset_resource = AddressableUserQuerySetResource(mock_orm.QuerySet(
            User(pk=1, name='Alice', age=31),
            User(pk=2, name='Bob', age=20)
        ))

        model_resource = queryset_resource.get_child_resource(mock_context(), 999)
        self.assertIsNone(model_resource)


class ResourcePrepareTest(unittest.TestCase):
    class TestResource(resources.ModelResource):
        model_class = User
        fields = [
            fields.AttributeField(attribute='group.name', type=str),
            fields.AttributeField(attribute='domain.name', type=str)
        ]

    def test_prepare(self):
        related = resources.Related()

        queryset = self.TestResource.prepare(mock_context(), related)
        self.assertEqual(queryset._select, {
            'group',
            'domain'
        })

    def test_prepare_optional(self):
        class NoopField(object):
            def handle_incoming(self, ctx, source_dict, target_obj):
                pass

            def handle_outgoing(self, ctx, source_obj, target_dict):
                pass

        class TestResource(resources.ModelResource):
            model_class = User
            fields = [
                NoopField(),
            ]

        related = resources.Related()
        # Will not raise an error
        TestResource.prepare(mock_context(), related)

    def test_prepere_after_filter(self):
        """
        Django will reset related selects when a filter is added
        """
        queryset = MagicMock()
        queryset_resource = ComplexUserResourceQuerySetResource(queryset)

        queryset_resource.get(mock_context(), EmptyParams())
        calls = call.all().distinct().filter().select_related('manager').prefetch_related('reports').call_list()
        queryset.assert_has_calls(calls)


class DjangoUserResource(resources.ModelResource):
    '''
    Exists to test SchemaResource using Django's User model
    '''
    model_class = DjangoUser
    fields = [
        fields.AttributeField('date_joined', type=datetime),
    ]
    # TODO add filtering and sort order


class SchemaResourceTest(unittest.TestCase):

    def setUp(self):
        self.json_formatter = formatters.JSONFormatter()

    def do_assert_date_equal(self, key):
        field = self.do_get()['fields'][key]
        field['default'] = date_str

        self.assertDictEqual(field, user_resource_schema['fields'][key])

    def do_assert_equal(self, key, assert_type=''):
        getattr(self, 'assert{}Equal'.format(assert_type))(self.do_get()[key], user_resource_schema[key])

    def do_assert_field_equal(self, key):
        self.assertDictEqual(self.do_get()['fields'][key], user_resource_schema['fields'][key])

    def do_get(self):
        """
        Make GET request, return response
        """
        resource = resources.SchemaResource(DjangoUserResource)
        ctx = mock_context()
        ctx.build_resource_uri = lambda resource: 'uri://user/schema/'
        return resource.get(ctx)

    def test_get(self):
        """
        Test basic GET request for a SchemaResource
        """
        self.assertIsInstance(self.do_get(), dict)

    def test_allowed_detail_http_methods(self):
        self.do_assert_equal('allowedDetailHttpMethods', assert_type='List')

    def test_allowed_list_http_methods(self):
        self.do_assert_equal('allowedListHttpMethods', assert_type='List')

    def test_default_format(self):
        self.do_assert_equal('defaultFormat')

    def test_default_limit(self):
        self.do_assert_equal('defaultLimit')

    def test_filtering(self):
        self.do_assert_equal('filtering', assert_type='Dict')

    def test_ordering(self):
        self.do_assert_equal('ordering', assert_type='List')

    def test_resource_uri(self):
        self.do_assert_equal('resourceUri')

    def test_fields_length(self):
        self.assertEqual(len(self.do_get()['fields']), len(user_resource_schema['fields']))

    def test_field_date_joined(self):
        self.do_assert_date_equal('dateJoined')



import logging
import unittest
import mock
from savory_pie.django.utils import Related, getLogger
from savory_pie.tests.django import mock_orm


class LoggerTestCase(unittest.TestCase):

    def test_logger_callable(self):
        with self.assertRaises(AssertionError):
            logger = getLogger()
            logger.callable('some var', logger=None)

    def test_logger_im_func(self):
        func = mock.MagicMock(return_value=None, name='logger')

        im_func = mock.MagicMock(name='im_func')

        func_code = mock.MagicMock(name='func_code')
        im_func.func_code = func_code
        func.im_func = im_func

        func_code.co_filename = 'file name'
        func_code.co_firstlineno = 23
        func_code.co_name = 'name'

        logger = mock.MagicMock()

        lg = getLogger()
        lg.callable(func, logger=logger)
        logger._log.assert_called_with(logging.DEBUG, 'file name:23 name', [], {})

    def test_logger(self):
        func = mock.MagicMock(return_value=None, name='logger', spec=['func_code'])

        func_code = mock.MagicMock(name='func_code')
        func.func_code = func_code

        func_code.co_filename = 'file name'
        func_code.co_firstlineno = 23
        func_code.co_name = 'name'

        logger = mock.MagicMock()

        lg = getLogger()
        lg.callable(func, logger=logger)
        logger._log.assert_called_with(logging.DEBUG, 'file name:23 name', [], {})

    def test_logger_alert(self):
        logger = mock.MagicMock()

        lg = getLogger()
        lg.alert('object', marker='1', logger=logger)
        bannerEdge = '1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1'

        calls = [
            mock.call(logging.DEBUG, bannerEdge, [], {}),
            mock.call(logging.DEBUG, bannerEdge, [], {}),
            mock.call(logging.DEBUG, '                                     object', [], {}),
            mock.call(logging.DEBUG, bannerEdge, [], {}),
            mock.call(logging.DEBUG, bannerEdge, [], {}),

        ]
        logger._log.assert_has_calls(calls)

    @mock.patch('savory_pie.django.utils.pprint.pformat')
    def test_pp_enabled(self, pformat):
        pformat.return_value = 'something pprinted'
        logger = mock.MagicMock()
        logger.isEnabledFor.return_value = True
        lg = getLogger()
        lg.pprint('object', logger=logger)
        logger._log.assert_called_with(logging.DEBUG, '\nsomething pprinted', [], {})

    @mock.patch('savory_pie.django.utils.pprint.pformat')
    def test_pp_dis_enabled(self, pformat):
        pformat.return_value = 'something pprinted'
        logger = mock.MagicMock()
        logger.isEnabledFor.return_value = False
        lg = getLogger()
        lg.pprint('object', logger=logger)
        self.assertFalse(logger._log.called)

    def test_logger_before_disabled(self):
        logger = mock.MagicMock()
        logger.isEnabledFor.return_value = False
        lg = getLogger()
        lg.before_queries(txt='object', logger=logger)
        self.assertFalse(logger._log.called)

    @mock.patch('savory_pie.django.utils.connection')
    def test_logger_before_queries(self, connection):
        connection.queries = [1, 2]
        logger = mock.MagicMock()
        logger.isEnabledFor.return_value = True
        lg = getLogger()
        lg.before_queries(txt='object', logger=logger)
        logger._log.assert_called_with(logging.DEBUG, 'object', [], {})
        self.assertEqual(logger._num_queries, 2)

    @mock.patch('savory_pie.django.utils.connection')
    def test_logger_before_queries_notxt(self, connection):
        connection.queries = [1, 2]
        logger = mock.MagicMock()
        logger.isEnabledFor.return_value = True
        lg = getLogger()
        lg.before_queries(logger=logger)
        self.assertFalse(logger._log.called)
        self.assertEqual(logger._num_queries, 2)

    @mock.patch('savory_pie.django.utils.connection')
    def test_logger_before_queries_disabled(self, connection):
        connection.queries = [1, 2]
        logger = mock.MagicMock()
        logger._num_queries = 0
        logger.isEnabledFor.return_value = False
        lg = getLogger()
        lg.after_queries(logger=logger)
        self.assertFalse(logger._log.called)
        self.assertEqual(logger._num_queries, 0)

    @mock.patch('savory_pie.django.utils.connection')
    def test_logger_after_queries_enabled(self, connection):
        logger = mock.MagicMock()
        logger._num_queries = 1
        qry1 = {'name': 'qry1', 'time': '1', 'sql': 'select'}
        qry2 = {'name': 'qry2', 'time': '2', 'sql': 'insert'}
        qry3 = {'name': 'qry3', 'time': '3', 'sql': 'update'}

        connection.queries = [qry1, qry2, qry3]
        logger.isEnabledFor.return_value = True

        lg = getLogger()
        lg.after_queries(obj='object', logger=logger)

        calls = [
            mock.call(logging.DEBUG, 'Database hits: 2', [], {}),
            mock.call(logging.DEBUG, 'Database hit, 2 seconds\ninsert', [], {}),
            mock.call(logging.DEBUG, 'Database hit, 3 seconds\nupdate', [], {}),
        ]
        logger._log.assert_has_calls(calls)


class RelatedTest(unittest.TestCase):
    def test_select(self):
        related = Related()

        related.select('foo')
        self.assertEqual(related._select, {
            'foo'
        })

        related.select('bar')
        self.assertEqual(related._select, {
            'foo',
            'bar'
        })

    def test_prefetch(self):
        related = Related()

        related.prefetch('foo')
        self.assertEqual(related._prefetch, {
            'foo'
        })

        related.prefetch('bar')
        self.assertEqual(related._prefetch, {
            'foo',
            'bar'
        })

    def test_sub_select(self):
        related = Related()
        sub_related = related.sub_select('foo')

        sub_related.select('bar')
        sub_related.prefetch('baz')

        self.assertEqual(related._select, {
            'foo__bar'
        })
        self.assertEqual(related._prefetch, {
            'foo__baz'
        })

    def test_sub_prefetch(self):
        related = Related()
        sub_related = related.sub_prefetch('foo')

        sub_related.select('bar')
        sub_related.prefetch('baz')

        self.assertEqual(related._select, set())
        # Because foo is assumed to have a non-one cardinality, sub-selects
        # through foo are also converted into prefetch-es.  In this case, bar.
        self.assertEqual(related._prefetch, {
            'foo__bar',
            'foo__baz'
        })

    def test_sub_prefetch_continuation(self):
        related = Related()
        sub_related = related.sub_prefetch('foo')
        sub_sub_related = sub_related.sub_select('bar')

        sub_sub_related.select('baz')

        # Because foo was prefetch, the sub-select of bar is also forced into
        # prefetch mode, so foo__bar__baz ends up being prefetched.
        self.assertEqual(related._prefetch, {
            'foo__bar__baz'
        })

    def test_empty_prepare(self):
        related = Related()

        queryset = related.prepare(mock_orm.QuerySet())

        self.assertEqual(queryset._selected, set())
        self.assertEqual(queryset._prefetched, set())

    def test_prepare(self):
        related = Related()

        related.select('foo')
        related.prefetch('bar')

        queryset = related.prepare(mock_orm.QuerySet())

        self.assertEqual(queryset._selected, {
            'foo'
        })
        self.assertEqual(queryset._prefetched, {
            'bar'
        })



from exceptions import Exception


class AuthorizationError(Exception):

    def __init__(self, name='', *args, **kwargs):
        self.name = name
        super(AuthorizationError, self).__init__(*args, **kwargs)


class MethodNotAllowedError(Exception):

    def __init__(self, method='', *args, **kwargs):
        self.method = method
        super(MethodNotAllowedError, self).__init__(*args, **kwargs)


class PreConditionError(Exception):
    pass


class ResourceNotFoundError(Exception):
    pass


class SavoryPieError(Exception):
    """
    General Savory Pie Error
    """
    pass
